"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var TestHelper_1 = require("./TestHelper");
describe('noCommentedOutCodeRule', function () {
    var ruleName = 'no-commented-out-code';
    it('should pass on single word', function () {
        var script = "\n            // todo\n        ";
        TestHelper_1.TestHelper.assertViolations(ruleName, script, []);
    });
    it('should pass on single line', function () {
        var script = "\n            // todo: add passing example\n        ";
        TestHelper_1.TestHelper.assertViolations(ruleName, script, []);
    });
    it('should pass on long single line', function () {
        var script = "\n            // todo: add failing example and update assertions\n        ";
        TestHelper_1.TestHelper.assertViolations(ruleName, script, []);
    });
    it('should pass on long single line', function () {
        var script = "\n        /**\n         * Implementation of the no-commented-out-code rule.\n         */\n        ";
        TestHelper_1.TestHelper.assertViolations(ruleName, script, []);
    });
    it('should pass on long single line', function () {
        var script = "\n        /**\n        Beautifier\n        */\n        ";
        TestHelper_1.TestHelper.assertViolations(ruleName, script, []);
    });
    it('should pass on tslint:disable comment', function () {
        var script = "\n        // tslint:disable:no-reserved-keywords\n        ";
        TestHelper_1.TestHelper.assertViolations(ruleName, script, []);
    });
    it('should pass on tslint:enable comment', function () {
        var script = "\n        // tslint:enable:no-reserved-keywords\n        ";
        TestHelper_1.TestHelper.assertViolations(ruleName, script, []);
    });
    it('should pass on code with inline comments', function () {
        var script = "\n        const obj = {\n            ruleName: \"no-commented-out-code\",\n            type: \"maintainability\", // one of: 'functionality' | 'maintainability' | 'style' | 'typescript'\n            description: \"... add a meaningful one line description\",\n            options: null,\n            optionsDescription: \"\",\n            optionExamples: [], //Remove this property if the rule has no options\n            typescriptOnly: false,\n            issueClass: \"Non-SDL\", // one of: 'SDL' | 'Non-SDL' | 'Ignored'\n            issueType: \"Warning\", // one of: 'Error' | 'Warning'\n            severity: \"Low\", // one of: 'Critical' | 'Important' | 'Moderate' | 'Low'\n            level: \"Opportunity for Excellence\", // one of 'Mandatory' | 'Opportunity for Excellence'\n            group: \"Clarity\", // one of 'Ignored' | 'Security' | 'Correctness' | 'Clarity' | 'Whitespace' | 'Configurable' | 'Deprecated'\n          };\n        ";
        TestHelper_1.TestHelper.assertViolations(ruleName, script, []);
    });
    it('should fail on console.log', function () {
        var script = "\n            // console.log(\"hello world\");\n        ";
        TestHelper_1.TestHelper.assertViolations(ruleName, script, [
            {
                failure: 'No commented out code.',
                name: 'file.ts',
                ruleName: 'no-commented-out-code',
                ruleSeverity: 'ERROR',
                startPosition: {
                    character: 13,
                    line: 2
                }
            }
        ]);
    });
});
//# sourceMappingURL=NoCommentedOutCodeRuleTests.js.map